package demo;

import java.awt.BasicStroke;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.Paint;
import java.awt.Stroke;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.geom.Point2D;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JApplet;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.JToggleButton;
import javax.swing.border.TitledBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import org.apache.commons.collections15.Transformer;
import org.apache.commons.collections15.functors.ConstantTransformer;
import org.apache.commons.collections15.functors.MapTransformer;
import org.apache.commons.collections15.map.LazyMap;

import tr.edu.javaclassranking.Edge;
import tr.edu.javaclassranking.GraphBuilder;
import tr.edu.javaclassranking.Vertex;

import edu.uci.ics.jung.algorithms.cluster.EdgeBetweennessClusterer;
import edu.uci.ics.jung.algorithms.layout.AggregateLayout;
import edu.uci.ics.jung.algorithms.layout.CircleLayout;
import edu.uci.ics.jung.algorithms.layout.FRLayout;
import edu.uci.ics.jung.algorithms.layout.Layout;
import edu.uci.ics.jung.algorithms.layout.util.Relaxer;
import edu.uci.ics.jung.graph.DirectedSparseMultigraph;
import edu.uci.ics.jung.graph.Graph;
import edu.uci.ics.jung.graph.SparseMultigraph;
import edu.uci.ics.jung.visualization.GraphZoomScrollPane;
import edu.uci.ics.jung.visualization.VisualizationViewer;
import edu.uci.ics.jung.visualization.control.DefaultModalGraphMouse;

public class ClusteringDemo extends JApplet {


	VisualizationViewer<Vertex, Edge> vv;
	private static final long serialVersionUID = -7768774839144970339L;

	@SuppressWarnings({ "unchecked", "rawtypes" })
	Map<Vertex,Paint> vertexPaints = 
	LazyMap.<Vertex,Paint>decorate(new HashMap<Vertex,Paint>(),
			new ConstantTransformer(Color.white));
	@SuppressWarnings({ "unchecked", "rawtypes" })
	Map<Edge,Paint> edgePaints =
	LazyMap.<Edge,Paint>decorate(new HashMap<Edge,Paint>(),
			new ConstantTransformer(Color.blue));

	public final Color[] similarColors =
		{
			new Color(216, 134, 134),
			new Color(135, 137, 211),
			new Color(134, 206, 189),
			new Color(206, 176, 134),
			new Color(194, 204, 134),
			new Color(145, 214, 134),
			new Color(133, 178, 209),
			new Color(103, 148, 255),
			new Color(60, 220, 220),
			new Color(30, 250, 100)
		};




	public static void main(String[] args) {

		ClusteringDemo cd = new ClusteringDemo();
		cd.start();
		
		JFrame jf = new JFrame();
		jf.getContentPane().add(cd);
		
		jf.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		jf.pack();
		jf.setVisible(true);

	}

	public void start() {

		setUpView();

	}

	private void setUpView() {

		Graph<Vertex, Edge> g = new DirectedSparseMultigraph<Vertex, Edge>();
		@SuppressWarnings("unused")
		GraphBuilder gb = new GraphBuilder(g, "java.db", 2);

		final AggregateLayout<Vertex, Edge> layout = new AggregateLayout<Vertex, Edge>(new FRLayout<Vertex,Edge>(g));
		vv = new VisualizationViewer<Vertex, Edge>(layout);
		vv.setBackground( Color.white );

		vv.getRenderContext().setVertexFillPaintTransformer(MapTransformer.<Vertex,Paint>getInstance(vertexPaints));
		vv.getRenderContext().setVertexDrawPaintTransformer(new Transformer<Vertex,Paint>() {
			public Paint transform(Vertex v) {
				if(vv.getPickedVertexState().isPicked(v)) {
					return Color.cyan;
				} else {
					return Color.BLACK;
				}
			}
		});

		vv.getRenderContext().setEdgeDrawPaintTransformer(MapTransformer.<Edge,Paint>getInstance(edgePaints));

		vv.getRenderContext().setEdgeStrokeTransformer(new Transformer<Edge,Stroke>() {
			protected final Stroke THIN = new BasicStroke(1);
			protected final Stroke THICK= new BasicStroke(2);
			public Stroke transform(Edge e)
			{
				Paint c = edgePaints.get(e);
				if (c == Color.LIGHT_GRAY)
					return THIN;
				else 
					return THICK;
			}
		});

		//add restart button
		JButton scramble = new JButton("Restart");
		scramble.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				Layout<Vertex, Edge> layout = vv.getGraphLayout();
				layout.initialize();
				Relaxer relaxer = vv.getModel().getRelaxer();
				if(relaxer != null) {
					relaxer.stop();
					relaxer.prerelax();
					relaxer.relax();
				}
			}

		});


		DefaultModalGraphMouse<Vertex, Edge> gm = new DefaultModalGraphMouse<Vertex, Edge>();
		vv.setGraphMouse(gm);

		final JToggleButton groupVertices = new JToggleButton("Group Clusters");

		//Create slider to adjust the number of edges to remove when clustering
		final JSlider edgeBetweennessSlider = new JSlider(JSlider.HORIZONTAL);
		edgeBetweennessSlider.setBackground(Color.WHITE);
		edgeBetweennessSlider.setPreferredSize(new Dimension(210, 50));
		edgeBetweennessSlider.setPaintTicks(true);
		edgeBetweennessSlider.setMaximum(g.getEdgeCount());
		edgeBetweennessSlider.setMinimum(0);
		edgeBetweennessSlider.setValue(0);
		edgeBetweennessSlider.setMajorTickSpacing(10);
		edgeBetweennessSlider.setPaintLabels(true);
		edgeBetweennessSlider.setPaintTicks(true);

		//		edgeBetweennessSlider.setBorder(BorderFactory.createLineBorder(Color.black));
		//TO DO: edgeBetweennessSlider.add(new JLabel("Node Size (PageRank With Priors):"));
		//I also want the slider value to appear
		final JPanel eastControls = new JPanel();
		eastControls.setOpaque(true);
		eastControls.setLayout(new BoxLayout(eastControls, BoxLayout.Y_AXIS));
		eastControls.add(Box.createVerticalGlue());
		eastControls.add(edgeBetweennessSlider);

		final String COMMANDSTRING = "Edges removed for clusters: ";
		final String eastSize = COMMANDSTRING + edgeBetweennessSlider.getValue();

		final TitledBorder sliderBorder = BorderFactory.createTitledBorder(eastSize);
		eastControls.setBorder(sliderBorder);
		//eastControls.add(eastSize);
		eastControls.add(Box.createVerticalGlue());

		groupVertices.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				clusterAndRecolor(layout, edgeBetweennessSlider.getValue(), 
						similarColors, e.getStateChange() == ItemEvent.SELECTED);
				vv.repaint();
			}});

		clusterAndRecolor(layout, 0, similarColors, groupVertices.isSelected());

		edgeBetweennessSlider.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
				JSlider source = (JSlider) e.getSource();
				if (!source.getValueIsAdjusting()) {
					int numEdgesToRemove = source.getValue();
					clusterAndRecolor(layout, numEdgesToRemove, similarColors,
							groupVertices.isSelected());
					sliderBorder.setTitle(
							COMMANDSTRING + edgeBetweennessSlider.getValue());
					eastControls.repaint();
					vv.validate();
					vv.repaint();
				}
			}
		});

		Container content = getContentPane();
		content.add(new GraphZoomScrollPane(vv));
		JPanel south = new JPanel();
		JPanel grid = new JPanel(new GridLayout(2,1));
		grid.add(scramble);
		grid.add(groupVertices);
		south.add(grid);
		south.add(eastControls);
		JPanel p = new JPanel();
		p.setBorder(BorderFactory.createTitledBorder("Mouse Mode"));
		p.add(gm.getModeComboBox());
		south.add(p);
		content.add(south, BorderLayout.SOUTH);

	}

	public void clusterAndRecolor(AggregateLayout<Vertex,Edge> layout,
			int numEdgesToRemove,
			Color[] colors, boolean groupClusters) {
		//Now cluster the vertices by removing the top 50 edges with highest betweenness
		//		if (numEdgesToRemove == 0) {
		//			colorCluster( g.getVertices(), colors[0] );
		//		} else {

		Graph<Vertex,Edge> g = layout.getGraph();
		layout.removeAll();

		EdgeBetweennessClusterer<Vertex,Edge> clusterer =
				new EdgeBetweennessClusterer<Vertex,Edge>(numEdgesToRemove);
		Set<Set<Vertex>> clusterSet = clusterer.transform(g);
		List<Edge> edges = clusterer.getEdgesRemoved();

		int i = 0;
		//Set the colors of each node so that each cluster's vertices have the same color
		for (Iterator<Set<Vertex>> cIt = clusterSet.iterator(); cIt.hasNext();) {

			Set<Vertex> vertices = cIt.next();
			Color c = colors[i % colors.length];

			colorCluster(vertices, c);
			if(groupClusters == true) {
				groupCluster(layout, vertices);
			}
			i++;
		}
		for (Edge e : g.getEdges()) {

			if (edges.contains(e)) {
				edgePaints.put(e, Color.lightGray);
			} else {
				edgePaints.put(e, Color.black);
			}
		}

	}

	private void colorCluster(Set<Vertex> vertices, Color c) {
		for (Vertex v : vertices) {
			vertexPaints.put(v, c);
		}
	}
	
	private void groupCluster(AggregateLayout<Vertex,Edge> layout, Set<Vertex> vertices) {
		if(vertices.size() < layout.getGraph().getVertexCount()) {
			Point2D center = layout.transform(vertices.iterator().next());
			Graph<Vertex,Edge> subGraph = SparseMultigraph.<Vertex,Edge>getFactory().create();
			for(Vertex v : vertices) {
				subGraph.addVertex(v);
			}
			Layout<Vertex,Edge> subLayout = 
				new CircleLayout<Vertex,Edge>(subGraph);
			subLayout.setInitializer(vv.getGraphLayout());
			subLayout.setSize(new Dimension(40,40));

			layout.put(subLayout,center);
			vv.repaint();
		}
	}
	


}
