package demo;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.PrintWriter;
import java.util.Hashtable;

import org.apache.commons.collections15.Transformer;

import tr.edu.javaclassranking.Edge;
import tr.edu.javaclassranking.Vertex;
import tr.edu.javaclassranking.analyzers.GetInformation;
import edu.uci.ics.jung.algorithms.scoring.DegreeScorer;
import edu.uci.ics.jung.graph.DirectedSparseMultigraph;
import edu.uci.ics.jung.graph.Graph;

public class OldAnalyze {
	private Graph<Vertex, Edge> g = new DirectedSparseMultigraph<Vertex, Edge>();
	private Hashtable<Short, Vertex> ht = null;
	private SimpleTransformer st = null;
	private double alpha = 2.2;

	private static final String doubleFormat = "%.3f\t%s\r\n";
	private static final String intFormat = "%d\t%s\r\n";


	public void analyze() {

		System.out.println("Vertex #: " + g.getVertexCount());
		System.out.println("Edge   #: " + g.getEdgeCount());

		PWriter pw = null;
		try {

			st = new SimpleTransformer();

			pw = new PWriter("DegreeScorer");
			DegreeScorer<Vertex> ds = new DegreeScorer<Vertex>(g);
			for (short i=1; i<=g.getVertexCount(); i++) {
				pw.printf(intFormat, ds.getVertexScore(ht.get(i)), new GetInformation("").getPackageAndClassName(i, "") );
			}
			ds = null;
			System.out.println("DegreeScorer");
			pw.close();

		} catch (Exception e) {
			e.printStackTrace();
		}

		analyzeHelper("BarycenterScorer");
		analyzeHelper("BetweennessCentrality");
		analyzeHelper("ClosenessCentrality");
		analyzeHelper("DistanceCentralityScorer");
		analyzeHelper("EigenvectorCentrality");
		analyzeHelper("PageRank");

	}

	private void analyzeHelper(String name) {

		PWriter pw = null;

		try {
			pw = new PWriter(name);

			Class<?> klass = Class.forName( "edu.uci.ics.jung.algorithms.scoring." + name );
			Class<?> grp = Class.forName( "edu.uci.ics.jung.graph.Graph" );
			Class<?> hypergrp = Class.forName( "edu.uci.ics.jung.graph.Hypergraph" );
			Class<?> trn = Class.forName ("org.apache.commons.collections15.Transformer");
			Class<?> obj = Class.forName("java.lang.Object");
			Class<?> bool = obj.getMethod("equals", obj).getReturnType();
			Class<?> dbl = Class.forName("java.lang.Double").getMethod("doubleValue", (Class<?>[])null).getReturnType();

			Object o = null;
			try {
				o = klass.getConstructor(grp, trn).newInstance(g, st);
			} catch (Exception e ) {
				try {
					o = klass.getConstructor(hypergrp, trn).newInstance(g, st);
				} catch (Exception e2) {
					try {
						o = klass.getConstructor(hypergrp, trn, bool).newInstance(g, st, true);
					} catch (Exception e3) {
						try {
							o = klass.getConstructor(hypergrp, trn, dbl).newInstance(g, st, alpha);
						} catch (Exception e4) {
							e4.printStackTrace();
						}
					}
				}
			}

			for (short i=1; i<=g.getVertexCount(); i++) {
				pw.printf(doubleFormat, klass.getMethod("getVertexScore", obj).invoke(o, ht.get(i)), new GetInformation("").getPackageAndClassName(i,"") );
			}		
			System.out.println(name);
			pw.close();

		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	class PWriter extends PrintWriter {
		public PWriter(String fileName) throws FileNotFoundException {
			super(new File ( "analyze/" + fileName + ".txt"));
		}
	}
	
	class SimpleTransformer implements Transformer<Edge, Short> {
		@Override
		public Short transform(Edge e) {
			return e.getWeight();
		}
	}
	
}
