package tr.edu.javaclassranking;

import java.io.File;
import java.io.FileInputStream;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.List;
import java.util.Scanner;
import java.util.StringTokenizer;
import java.util.jar.JarEntry;
import java.util.jar.JarInputStream;

public class DatabaseBuilder {

	private String JAR_LOC = "C:\\Users\\D7\\Documents\\JavaWorkspaces\\Libraries\\";
	private String prefixForClasses = null;
	private String dbName = null;

	private Connection conn = null;

	private static String schema = "schema.txt";

	private int methodID = 0;
	private long counter = 0;
	private boolean done = false;
	private short[][] numberOfRefers;

	public DatabaseBuilder (String jarName, String outputName, String pref) {
		JAR_LOC += jarName + ".jar";
		dbName = outputName + ".db";
		prefixForClasses = pref;
	}

	public void start() {

//		new Timer().start();

		List<Class<?>> classes = null;
		buildDB();
		classes = getClasses(JAR_LOC, prefixForClasses );

		//		 String and Integer are place holders here.
		//		 first row:	a method in String returning Integer.
		//		 second row:	a method in String needs Integer as a parameter.
		numberOfRefers = new short[2][classes.size() + 10 ];

		insertPrimitives();
		insertClasses(classes);
		insertModifiers();
		insertMethodsModifiersParams(classes);
		done = true;
		
		System.out.println("Database is filled.\n# of classes: " + (classes.size()+9) + " (including (#9) primitives)\n");
	}

	private void insertMethodsModifiersParams(List<Class<?>> classes) {

		try {
			Class.forName("org.sqlite.JDBC");
			conn = DriverManager.getConnection("jdbc:sqlite:" + dbName);
			conn.setAutoCommit(false);

			PreparedStatement prep = conn.prepareStatement("insert into method values (?,?,?,?);");
			PreparedStatement prep2 = conn.prepareStatement("insert into method_modifier values (?,?);");
			PreparedStatement prep3 = conn.prepareStatement("insert into param values (?,?,?);");

			for ( Class<?> i : classes ) {

				Method methods[] = i.getMethods();

				//id, name, return, owner
				int idOfTheClass = getClassID(i);
				prep.setInt(4, idOfTheClass);

				for ( Method j : methods ) {

					//insert method.
					int jReturnTypeID = (getClassID(j.getReturnType()));
					if ( jReturnTypeID == -1 ) continue;
					prep.setInt(1, ++methodID);
					prep.setString(2, j.getName());
					prep.setInt(3, jReturnTypeID );
					prep.executeUpdate();
					counter++;

					numberOfRefers[0][jReturnTypeID]++;

					//inserting method's modifiers
					StringTokenizer st = new StringTokenizer (Modifier.toString(j.getModifiers()));
					prep2.setInt(1, methodID);
					while (st.hasMoreTokens()) {
						prep2.setString(2, st.nextToken());
						prep2.executeUpdate();
						counter++;
					}

					//inserting method's parameters
					int rank = 0;
					Class<?>[] parameters = j.getParameterTypes();
					prep3.setInt(1, methodID);
					for ( Class<?> k : parameters ) {
						prep3.setInt(2, ++rank);
						int kClassID = getClassID(k);
						if ( kClassID == -1)
							kClassID = 9;	//9 is void. You better search for the id of void.
						prep3.setInt(3, kClassID);
						prep3.executeUpdate();
						counter++;

						numberOfRefers[1][kClassID]++;
					}

				}
			}
			conn.commit();
			prep.close();
			prep2.close();
			prep3.close();
			conn.close();
//			System.out.println("Methods, Method's modifiers, Method's parameters are inserted");

		} catch (Exception e) {
			e.printStackTrace();
		}

	}

	private int getClassID (Class<?> klass) {

		try {

			PreparedStatement prep = conn.prepareStatement("select id from unit where name = ? and package = ?;");

			while ( klass.isArray() )
				klass = klass.getComponentType();
			prep.setString(1, klass.getName());
			if ( klass.isPrimitive() )
				prep.setString(2, "primitive");
			else prep.setString(2, klass.getPackage().getName());

			ResultSet rs = prep.executeQuery();
			//			counter++;

			int toReturn;
			if ( rs.next() ) {
				toReturn = rs.getInt("id");
			} else {	//not exist in the database.
				//				System.out.println("No Way! " + klass.getName() );
				toReturn = -1;
			}
			rs.close();
			prep.close();
			return toReturn;
		} catch (SQLException e) {
			e.printStackTrace();
			return -1;
		}

	}

	private void insertModifiers() {

		String[] modifiers = {
				"public",
				"protected",
				"private",
				"abstract",
				"static",
				"final",
				"transient",
				"volatile",
				"synchronized",
				"native",
				"strictfp",
				"interface"
		};

		try {
			Class.forName("org.sqlite.JDBC");
			conn = DriverManager.getConnection("jdbc:sqlite:" + dbName);

			PreparedStatement prep = conn.prepareStatement("insert into modifier values (?);");
			conn.setAutoCommit(false);

			for ( String string : modifiers ) {
				prep.setString(1, string);
				prep.executeUpdate(); counter++;
			}
			conn.commit();
			prep.close();
			conn.close();
//			System.out.println("Modifiers are inserted");
		} catch (Exception e) {
			e.printStackTrace();
		}

	}

	private void insertClasses(List<Class<?>> classes) {

		try {
			Class.forName("org.sqlite.JDBC");
			conn = DriverManager.getConnection("jdbc:sqlite:" + dbName);
			conn.setAutoCommit(false);

			PreparedStatement prep = conn.prepareStatement("insert into unit values (?,?,?);");
			for ( Class<?> i : classes ) {
				prep.setString(2, i.getName());
				prep.setString(3, i.getPackage().getName());
				prep.executeUpdate(); counter++;
				
			}
			conn.commit();
			prep.close();
			conn.close();
//			System.out.println("Classes are inserted");
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	private void insertPrimitives() {

		String[] primitives = {
				"boolean",
				"byte",
				"char",
				"short",
				"int",
				"long",
				"float",
				"double",
				"void"
		};

		String packagePrimitive = "primitive";

		try {
			Class.forName("org.sqlite.JDBC");
			conn = DriverManager.getConnection("jdbc:sqlite:" + dbName);
			PreparedStatement prep = conn.prepareStatement("insert into unit values (?,?,?);");
			conn.setAutoCommit(false);
			for ( String string : primitives ) {
				prep.setString(2, packagePrimitive + "." + string);
				prep.setString(3, packagePrimitive);
				prep.executeUpdate(); counter++;
			}
			conn.commit();
			prep.close();
			conn.close();
//			System.out.println("Primitives are inserted");
		} catch (Exception e) {
			e.printStackTrace();
		}

	}

	private List<Class<?>> getClasses (String jarLoc, String packagePrefix) {

		ArrayList<Class<?>> classes = new ArrayList<Class<?>>();
		packagePrefix = packagePrefix.replaceAll("\\." , "/");

		try {

			JarInputStream jarFile = new JarInputStream ( new FileInputStream (jarLoc));
			JarEntry jarEntry = jarFile.getNextJarEntry();

			while(true) {

				jarEntry = jarFile.getNextJarEntry();
				if(jarEntry == null) break;
				String name = jarEntry.getName();

				if( name.startsWith(packagePrefix) && !jarEntry.isDirectory() && name.endsWith(".class") ) {
					String file_path = name.replaceAll("/", "\\.").replaceAll(".class$", "" );
					Class<?> klass = Class.forName( file_path );
					classes.add(klass);
				}
			}

			jarFile.close();
//			System.out.println("Classes are retrieved");


		} catch (Exception e) {
			e.printStackTrace();
		}
		return classes;
	}

	private void buildDB() {

		try {

			//deleting existing database
			File f = new File (dbName);
			f.delete();

			//creating schema of database

			Class.forName("org.sqlite.JDBC");
			conn = DriverManager.getConnection("jdbc:sqlite:" + dbName);

			Scanner oku = new Scanner (new FileInputStream(schema) );
			while (oku.hasNextLine()) {
				Statement stat = conn.createStatement();
				stat.executeUpdate(oku.nextLine());
			}
			conn.close();
			System.out.println("DB is built.");

		} catch (Exception e) {
			e.printStackTrace();
		}

	}

	class Timer extends Thread {
		public void run() {
			try {
				int second = 0;
				long prev = 0;
				while (!done) {
					sleep(5000);
					System.out.println( "" + (second+=5) + ":\t" + (counter-prev) + "\t" + (prev=counter)  );
				}
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
		}
	}

}
