package tr.edu.javaclassranking;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.Statement;
import java.util.Hashtable;

import edu.uci.ics.jung.graph.Graph;

public class GraphBuilder {

	private Graph<Vertex, Edge> g = null;
	private String dbName = null;
	private Hashtable<Short, Vertex> ht = null;
	private Connection conn = null;
	
	/*
	 * directionType indicates the direction of edge between classes.
	 * directionType 1 means:
	 * 		B A.method(C)	A->B, C->A
	 * directionType 2 means:
	 * 		B A.method(C)	A->B, A->C
	 */
	private int directionType;

	public GraphBuilder(Graph<Vertex, Edge> g, String dbName, int directionType) {
		this.g = g;
		this.dbName = dbName + ".db";
		this.directionType = directionType;
	}

	public void start() {

		try {

			Class.forName("org.sqlite.JDBC");
			conn = DriverManager.getConnection("jdbc:sqlite:" + dbName);

			addVertices();
			addReturns();
			addParameters();

			System.out.println("Graph is built.");
			System.out.println("# of Vertices: " + g.getVertexCount());
			System.out.println("# of Edges: " + g.getEdgeCount());
			
			conn.close();
			
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	private void addVertices() {

		try {

			Statement st = conn.createStatement();
			ResultSet rs = st.executeQuery("select id from unit;");

			ht = new Hashtable<Short, Vertex>();

			short id;
			Vertex v;
			while ( rs.next() ) {
				id = rs.getShort(1);
				v = new Vertex(id);
				ht.put(id, v);
				if ( !g.addVertex(v) ) {
					System.out.println("Error: Can't Add a Vertex");
					System.exit(0);
				}
			}
			rs.close();
			st.close();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	private void addReturns() {

		try {
			String returnSql = "select count(*) as c, unit.id, return from unit, method where unit.id=method.owner group by unit.id, return;";
			Statement st = conn.createStatement();
			ResultSet rs = st.executeQuery(returnSql);

			Edge e = null;
			Vertex v1;
			Vertex v2;
			short count;
			short A;
			short B;

			while ( rs.next() ) {
				count = rs.getShort(1);
				A = rs.getShort(2);
				B = rs.getShort(3);
				e = new Edge(count, A, B, true);
				v1 = ht.get(A);
				v2 = ht.get(B);
				if ( v1 == null || v2 == null ) {
					System.out.println("Error: Null Vertex Pointer");
					System.exit(0);
				}
				if ( !g.addEdge(e, v1, v2) ) {
					System.out.println("Error: Can't Add an Edge");
					System.exit(0);
				}
			}
			rs.close();
			st.close();
		} catch (Exception e) {
			e.printStackTrace();
		}

	}

	private void addParameters() {
		try {
			String paramSql = "select count(*) as c, unit.id, param.unit_id from unit, method, param where unit.id=method.owner and method.id=param.method_id group by unit.id, param.unit_id;";
			paramSql = "select count(*) as c, method.owner, param.unit_id from method, param where method.id=method_id group by method.owner, param.unit_id";
			Statement st = conn.createStatement();
			ResultSet rs = st.executeQuery(paramSql);
			
			System.out.println("Parameter query worked.");
			
			Edge e = null;
			Vertex v1 = null;
			Vertex v2 = null;
			short count;
			short A;
			short B;
			while ( rs.next() ) {
				count = rs.getShort(1);
				A = rs.getShort(2);
				B = rs.getShort(3);
				
				if ( directionType == 1 ) {
					e = new Edge(count, B, A, false);
				} else if ( directionType == 2) {
					e = new Edge(count, A, B, false);
				}
				
				v1 = ht.get(A);
				v2 = ht.get(B);
				if ( v1 == null || v2 == null ) {
					System.out.println("Error: Null Vertex Pointer");
					System.exit(0);
				}
				if ( !g.addEdge(e, v1, v2) ) {
					System.out.println("Error: Can't Add an Edge");
					System.out.println(v1 + " to " + v2);
					System.out.println(g.containsVertex(v1));
					System.out.println(g.containsVertex(v2));
					System.out.println(g.containsEdge(e));
					
					System.out.println();
//					System.exit(0);
				}
			}
			rs.close();
			st.close();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

}