package tr.edu.javaclassranking;

import java.util.ArrayList;

import org.apache.commons.collections15.Transformer;

import tr.edu.javaclassranking.analyzers.BCAnalyze;
import tr.edu.javaclassranking.analyzers.CCAnalyze;
import tr.edu.javaclassranking.transformers.OneOverDegreeTransformer;
import tr.edu.javaclassranking.transformers.UniweightedTransformer;
import edu.uci.ics.jung.graph.DirectedSparseMultigraph;
import edu.uci.ics.jung.graph.Graph;
import edu.uci.ics.jung.graph.UndirectedSparseMultigraph;

public class Runner {

	private String typeOfGraph;
	private Graph<Vertex, Edge> g = null;
	private String target;
	private int directionType;

//	@SuppressWarnings("unused")
	public static void main(String[] args) {
		
		String library = "rt";
		String target = "j";
		int directionType = 1;

		DatabaseBuilder db = new DatabaseBuilder(library, target, target);
		db.start();

		Graph<Vertex, Edge> g = null;
		GraphBuilder gb = null;
		Runner r = null;

		g = new DirectedSparseMultigraph<Vertex, Edge>();
		gb = new GraphBuilder(g, target, directionType);
		gb.start();

		r = new Runner(g, "DirectedSparseMultigraph", target, directionType);
		r.analyze();
		System.out.println();
		
		g = new UndirectedSparseMultigraph<Vertex, Edge>();
		gb = new GraphBuilder(g, target, directionType);
		gb.start();
		
		r = new Runner(g, "UndirectedSparseMultigraph", target, directionType);
		r.analyze();
		System.out.println();

	}

	public Runner(Graph<Vertex, Edge> g, String typeOfGraph, String target, int directionType) {
		this.g = g;
		this.typeOfGraph = typeOfGraph;
		this.target = target;
		this.directionType = directionType;
	}
	
	@SuppressWarnings("unchecked")
	public void analyze() {

		//for each transformer, run each ranking algorithm.
		ArrayList<Object> trans = new ArrayList<Object>();
		trans.add(new UniweightedTransformer());
		trans.add(new OneOverDegreeTransformer());

		BCAnalyze bca = null;
		for ( Object element : trans ) {
			bca = new BCAnalyze(g, (Transformer<Edge, ? extends Number>) element, target);
			String fileName = target + "_" + typeOfGraph + "_directionType_" + directionType + "_" + bca.getClass().getSimpleName() + "_on_" + element.getClass().getSimpleName();
			bca.printFile(fileName);
			System.out.println(fileName + " is printed");
		}
		bca = null;
		 
		CCAnalyze cca = null;
		for ( Object element : trans ) {
			cca = new CCAnalyze(g, (Transformer<Edge, ? extends Number>) element, target);
			String fileName = target + "_" + typeOfGraph + "_directionType_" + directionType + "_" + cca.getClass().getSimpleName() + "_on_" + element.getClass().getSimpleName();
			cca.printFile(fileName);
			System.out.println(fileName + " is printed");
		}
		cca = null;
	}
	
}
